$!----------------------------------------------------
$!               BIMED
$!         WRITTEN BY STEVEN I. ALTCHULER, PH.D.
$!----------------------------------------------------
$!
$!   FORMAT:  BIMED [ (INPUTS) ( # SWITCHES) ( - COMMANDS) ]
$!
$!   FOR HELP, TYPE "BIMED -H"
$!
$!
$! THIS COMMAND WILL RUN A BMDP PROGRAM.  IT WILL
$!    *  INQUIRE FOR PARAMETERS
$!    *  MAKE ALL FILE ASSIGNMENTS
$!    *  ANNOTATE THE OUTPUT WITH THE FILE ASSIGNMENTS
$!
$! IT CAN RUN WITH THE FOLLOWING USER-SETTABLE OPTIONS
$!    *  SINGLE (D) OR MULTIPLE PROBLEMS
$!    *  RUN BMDP INTERACTIVELY (D) OR ON THE BATCH QUEUE
$!    *  RUN WITHOUT (D) OR WITH USER-SUPPLIED FORTRAN TRANSFORMATIONS
$!    *  EDIT AND EXECUTE (D) THE BMDP PROGRAM, OR ONLY EDIT IT
$!----------------------------------------------------
$!----------------------------------------------------
$! FILES USED:
$!
$! SYS$ERROR        FOR009     ERROR MESSAGES
$! RECORD.B01       FOR011     DATA FROM FOTRAN TO DCL
$! EDIT1.B01        FOR012     INFORMATION PARAGRAPH
$! EDIT2.B01        FOR013     /INP PARAGRAPH CHANGES
$! EDIT3.B01        FOR014     /SAV PARAGRAPH CHANGES
$! STATUS.B01       FOR015     TERMINATION STATUS
$! INPTSTR.B01      FOR016     DATA FROM DCL TO FORTRAN
$!                  FOR017     USER ASSIGNED INPUT FILE
$! UTBA.B01         FOR018     LIST OF LOGICAL ASSIGNMENTS
$!                               TO BE MADE, FROM FORTRAN TO DCL
$! BIMED.HLP        FOR019     HELP FILE
$! CONTROL.B01                 COPY OF BMDP COMMAND LANGUAGE FILE
$! BIMED.BAT                   FILE SUBMITTED TO BATCH JOB
$!----------------------------------------------------
$!        INITIATE PROGRAM, WRITE INPTSTR.B01
$!----------------------------------------------------
$ ON CONTROL_Y THEN GOTO TERMINATION
$ ON SEVERE_ERROR THEN GOTO TERMINATION
$ ON ERROR THEN CONTINUE
$ SPACE := " "
$ DIREC := "DBA0:[BMDP.OBJ]"
$ OPEN/WRITE INPTSTR INPTSTR.B01
$ IF "''P1'" .NES. "" THEN WRITE INPTSTR "''P1'"
$ IF "''P2'" .NES. "" THEN WRITE INPTSTR "''P2'"
$ IF "''P3'" .NES. "" THEN WRITE INPTSTR "''P3'"
$ IF "''P4'" .NES. "" THEN WRITE INPTSTR "''P4'"
$ IF "''P5'" .NES. "" THEN WRITE INPTSTR "''P5'"
$ IF "''P6'" .NES. "" THEN WRITE INPTSTR "''P6'"
$ IF "''P7'" .NES. "" THEN WRITE INPTSTR "''P7'"
$ IF "''P8'" .NES. "" THEN WRITE INPTSTR "''P8'"
$ CLOSE INPTSTR
$ ASSIGN/USER_MODE SYS$ERROR FOR009
$ ASSIGN/USER_MODE RECORD.B01 FOR011
$ ASSIGN/USER_MODE EDIT1.B01 FOR012
$ ASSIGN/USER_MODE EDIT2.B01 FOR013
$ ASSIGN/USER_MODE EDIT3.B01 FOR014
$ ASSIGN/USER_MODE STATUS.B01 FOR015
$ ASSIGN/USER_MODE INPTSTR.B01 FOR016
$ ASSIGN/USER_MODE UTBA.B01 FOR018
$ ASSIGN TT: FOR005
$ ASSIGN TT: FOR006
$!----------------------------------------------------
$!          RUN THE FORTRAN PROGRAM TO READ USER INPUTS
$!----------------------------------------------------
$ RUN DBA0:[BMDP]BIMED.EXE
$ OPEN/READ STATUS STATUS.B01
$ READ STATUS STATCODE
$ CLOSE STATUS
$ IF ("''STATCODE'" .EQS. "HALT!!") THEN GOTO TERMINATION
$!----------------------------------------------------
$!          READ THE OUTPUT FROM THE FORTRAN PROGRAM
$!----------------------------------------------------
$ OPEN/READ RECRDFILE RECORD.B01
$ READ RECRDFILE PROGRAM
$ READ RECRDFILE LINKFILE
$ READ RECRDFILE CMDFILE
$ READ RECRDFILE OUTFILE
$ READ RECRDFILE F10FILE
$ READ RECRDFILE F20FILE
$ READ RECRDFILE F30FILE
$ READ RECRDFILE OPERATION
$ CLOSE RECRDFILE
$ ON ERROR THEN GOTO TERMINATION
$ COPY 'CMDFILE CONTROL.B01
$ ON ERROR THEN GOTO TERMINATION
$ ON ERROR THEN CONTINUE
$!----------------------------------------------------
$!          EDIT CONTROL.B01
$!----------------------------------------------------
$ ASSIGN/USER_MODE NL: SYS$OUTPUT
$ IF ('OPERATION .GE. 5) THEN $ GOTO PREIFSTAT
$!----------------------------------------------------
$!          EDIT HERE IF THERE ARE SAVE FILES
$!----------------------------------------------------
$ EDIT/SOS CONTROL.B01
/MATCH
N1,^/^:*/*,1000
@EDIT1.B01
.^/^
@EDIT2.B01
.^/^
@EDIT3.B01
SI.,1^/^:*/*
EB
$ GOTO IFSTAT
$!----------------------------------------------------
$!          EDIT HERE IF THERE ARE NO SAVE FILES
$!----------------------------------------------------
$ PREIFSTAT:
$ EDIT/SOS CONTROL.B01
/MATCH
N1,^/^:*/*,1000
@EDIT1.B01
.^/^
@EDIT2.B01
SI.,1^/^:*/*
EB
$!----------------------------------------------------
$!          DECIDE WHAT TYPE OF OPERATION.
$!
$!     KEY:
$!         0      BMDP WITH SAVE FILES
$!         1      BMDPT WITH SAVE FILES
$!         2      EDIT-ONLY WITH SAVE FILES
$!         3      BMDP WITH SAVE FILES, SUBMITTED TO BATCH
$!         4      BMDPT WITH SAVE FILES, SUBMITTED TO BATCH
$!         5      BMDP WITHOUT SAVE FILES
$!         6      BMDPT WITHOUT SAVE FILES
$!         7      EDIT-ONLY WITHOUT SAVE FILES
$!         8      BMDP WITHOUT SAVE FILES, SUBMITTED TO BATCH
$!         9      BMDPT WITHOUT SAVE FILES, SUBMITTED TO BATCH
$!
$!----------------------------------------------------
$ IFSTAT:
$ IF ('OPERATION .EQ. 0) THEN $ GOTO OP0
$ IF ('OPERATION .EQ. 1) THEN $ GOTO OP1
$ IF ('OPERATION .EQ. 2) THEN $ GOTO OP2
$ IF ('OPERATION .EQ. 3) THEN $ GOTO OP3
$ IF ('OPERATION .EQ. 4) THEN $ GOTO OP4
$ IF ('OPERATION .EQ. 5) THEN $ GOTO OP0
$ IF ('OPERATION .EQ. 6) THEN $ GOTO OP1
$ IF ('OPERATION .EQ. 7) THEN $ GOTO OP2
$ IF ('OPERATION .EQ. 8) THEN $ GOTO OP3
$ IF ('OPERATION .EQ. 9) THEN $ GOTO OP4
$!----------------------------------------------------
$!          BMDP RUN INTERACTIVELY
$!----------------------------------------------------
$ OP0:
$ OPEN/READ UTBA UTBA.B01
$ ASL:  READ/END=EASL0 UTBA UNO
$ READ/END=EASL0 UTBA UNA
$ FORUNIT := "FOR0''UNO'"
$ ASSIGN/USER_MODE 'UNA' 'FORUNIT
$ GOTO ASL
$ EASL0:  CLOSE UTBA
$ DEASSIGN FOR005
$ ASSIGN CONTROL.B01 FOR005
$ DEASSIGN FOR006
$ ASSIGN 'OUTFILE FOR006
$ DDBMD := "DBA0:[BMDP.OBJ]"'PROGRAM'
$ XQT := 'PROGRAM'"TMP"
$ ON SEVERE_ERROR THEN GOTO TERMINATION
$ SET MESSAGE/NOFACILITY/NOIDENTIFICATION/NOSEVERITY/NOTEXT
$ LINK/EXE='XQT'/MAP='XQT'/BRIEF 'DDBMD',DBA0:[BMDP]BMDP.LIB/LIB
$ SET MESSAGE/FACILITY/IDENTIFICATION/SEVERITY/TEXT
$ ON SEVERE_ERROR THEN GOTO TERMINATION
$ RUN 'XQT
$ IF OUTFILE .EQS. "BIMED.TMP" THEN PRINT/DELETE BIMED.TMP
$ GOTO TERMINATION
$!----------------------------------------------------
$!          BMDPT RUN INTERACTIVELY
$!----------------------------------------------------
$ OP1:
$ OPEN/READ UTBA UTBA.B01
$ ASL:  READ/END=EASL1 UTBA UNO
$ READ/END=EASL1 UTBA UNA
$ FORUNIT := "FOR0''UNO'"
$ ASSIGN/USER_MODE 'UNA' 'FORUNIT
$ GOTO ASL
$ EASL1:  CLOSE UTBA
$ DEASSIGN FOR005
$ ASSIGN CONTROL.B01 FOR005
$ DEASSIGN FOR006
$ ASSIGN 'OUTFILE FOR006
$ DDBMD := "DBA0:[BMDP.OBJ]"'PROGRAM'
$ XQT := 'PROGRAM'"TMP"
$ ON SEVERE_ERROR THEN GOTO TERMINATION
$ LINK/EXE='XQT'/MAP='XQT'/BRIEF 'LINKFILE','DDBMD',DBA0:[BMDP]BMDP.LIB/LIB
$ ON SEVERE_ERROR THEN GOTO TERMINATION
$ RUN 'XQT
$ IF OUTFILE .EQS. "BIMED.TMP" THEN PRINT/DELETE BIMED.TMP
$ GOTO TERMINATION
$!----------------------------------------------------
$!          EDIT-ONLY
$!----------------------------------------------------
$ OP2:
$ IF ("''OUTFILE'" .EQS. "SYS$OUTPUT") THEN TYPE CONTROL.B01
$ IF ("''OUTFILE'" .EQS. "SYS$OUTPUT") THEN GOTO TERMINATION
$ RENAME CONTROL.B01 'OUTFILE
$ IF OUTFILE .EQS. "BIMED.TMP" THEN PRINT/DELETE BIMED.TMP
$ GOTO TERMINATION
$ OP3:
$!----------------------------------------------------
$!          BMDP RUN ON THE BATCH QUEUE
$!----------------------------------------------------
$ RENAME UTBA.B01 UTBA.B02
$ RENAME CONTROL.B01 CONTROL.B02
$ OPEN/WRITE BATCH BIMED.BAT
$ WB := "WRITE BATCH"
$ WB "$ SET DEFAULT ''F$DIRECTORY()'"
$ WB "$ OPEN/READ UTBA UTBA.B02"
$ WB "$ ASL:  READ/END=EASL UTBA UNO"
$ WB "$ READ/END=EASL UTBA UNA"
$ WB "$ FORUNIT := FOR0'UNO'"
$ WB "$ ASSIGN/USER_MODE 'UNA' 'FORUNIT'"
$ WB "$ GOTO ASL"
$ WB "$ EASL:  CLOSE UTBA"
$ WB "$ ASSIGN CONTROL.B02 FOR005"
$ WB "$ ASSIGN ''OUTFILE' FOR006"
$ WB "$ ASSIGN ''F10FILE' FOR010"
$ WB "$ ASSIGN ''F20FILE' FOR020"
$ WB "$ ASSIGN ''F30FILE' FOR030"
$ WB "$ DDBMD := ''DIREC'''PROGRAM'"
$ WB "$ XQT := ''PROGRAM'TMP"
$ WB "$ ON SEVERE_ERROR THEN GOTO I1"
$ WB "$ LINK/EXE = 'XQT'/MAP='XQT'/BRIEF 'DDBMD',DBA0:[BMDP]BMDP.LIB/LIB"
$ WB "$ ON SEVERE_ERROR THEN GOTO I1"
$ WB "$ RUN 'XQT'"
$ IF OUTFILE .EQS. "BIMED.TMP" THEN WRITE BATCH "$ PRINT/DELETE BIMED.TMP"
$ WB "$ I1:  DEL *.B02.*"
$ WB "$ DEL 'XQT'.*.*"
$ WB "$ DEL BIMED.BAT.*"
$ WB "$ EXIT"
$ CLOSE BATCH
$ SUBMIT BIMED.BAT
$ GOTO TERMINATION
$!----------------------------------------------------
$!          BMDPT RUN ON THE BATCH QUEUE
$!----------------------------------------------------
$ OP4:
$ RENAME UTBA.B01 UTBA.B02
$ RENAME CONTROL.B01 CONTROL.B02
$ OPEN/WRITE BATCH BIMED.BAT
$ WB := "WRITE BATCH"
$ WB "$ SET DEFAULT ''F$DIRECTORY()'"
$ WB "$ OPEN/READ UTBA UTBA.B02"
$ WB "$ ASL:  READ/END=EASL UTBA UNO"
$ WB "$ READ/END=EASL UTBA UNA"
$ WB "$ FORUNIT := FOR0'UNO'"
$ WB "$ ASSIGN/USER_MODE 'UNA' 'FORUNIT'"
$ WB "$ GOTO ASL"
$ WB "$ EASL:  CLOSE UTBA"
$ WB "$ ASSIGN CONTROL.B02 FOR005"
$ WB "$ ASSIGN ''OUTFILE' FOR006"
$ WB "$ ASSIGN ''F10FILE' FOR010"
$ WB "$ ASSIGN ''F20FILE' FOR020"
$ WB "$ ASSIGN ''F30FILE' FOR030"
$ WB "$ ASSIGN ''LINKFILE' LINKFILE"
$ WB "$ DDBMD := ''DIREC'''PROGRAM'"
$ WB "$ XQT := ''PROGRAM'TMP"
$ WB "$ ON SEVERE_ERROR THEN GOTO I1"
$ WB "$ LINK/EXE = 'XQT'/MAP='XQT'/BRIEF LINKFILE,'DDBMD',DBA0:[BMDP]BMDP.LIB/LIB"
$ WB "$ ON SEVERE_ERROR THEN GOTO I1"
$ WB "$ RUN 'XQT'"
$ IF OUTFILE .EQS. "BIMED.TMP" THEN WRITE BATCH "$ PRINT/DELETE BIMED.TMP"
$ WB "$ I1:  DEL *.B02.*"
$ WB "$ DEL 'XQT'.*.*"
$ WB "$ DEL BIMED.BAT.*"
$ WB "$ EXIT"
$ CLOSE BATCH
$ SUBMIT BIMED.BAT
$!----------------------------------------------------
$!          TERMINATION PROCEDURES
$!----------------------------------------------------
$ TERMINATION:
$ ON CONTROL_Y THEN CONTINUE
$ ON SEVERE_ERROR THEN CONTINUE
$ IF ("''XQT'" .NES. "") THEN DEL 'XQT'.*.*
$ E4:  ON SEVERE_ERROR THEN CONTINUE
$ CLOSE/ERROR=E1 RECORD
$ E1:  ON SEVERE_ERROR THEN CONTINUE
$ CLOSE/ERROR=E2 BATCH
$ E2:  ON SEVERE_ERROR THEN CONTINUE
$ CLOSE/ERROR=E3 UTBA
$ E3:  ON SEVERE_ERROR THEN CONTINUE
$ CLOSE/ERROR=E5 STATUS
$ E5:  ON SEVERE_ERROR THEN CONTINUE
$ DEL *.B01.*
$ DEASSIGN FOR005
$ ON ERROR THEN CONTINUE
$ DEASSIGN FOR006
$ ON ERROR THEN CONTINUE
$ WRITE SYS$OUTPUT " "
$ WRITE SYS$OUTPUT "***BIMED COMPLETED***"
$ WRITE SYS$OUTPUT " "
$ EXIT
